 /*
 *
 * LKM1638 Library
 *  Peter Barnes (AA1YY) March, 2013
 *
 */

#include	<avr/io.h>
#include	<util/delay.h>
#include	<avr/pgmspace.h>
#include 	<string.h>

// data command = 0X40, display command = 0x80, address command = 0XC0

// SET YOUR PORT HERE
#define DIO_IN		DDRD&=~(1<<PD1)
#define DIO_OUT		DDRD|=(1<<PD1)
#define	DIO_READ	PIND&(1<<PD1)

// SET YOUR PINS HERE
#define DIO_high	PORTD|=(1<<PD1)
#define DIO_low		PORTD&=~(1<<PD1)
#define CLK_high	PORTD|=(1<<PD0)
#define CLK_low		PORTD&=~(1<<PD0)
#define STB_high	PORTD|=(1<<PD2)
#define STB_low		PORTD&=~(1<<PD2)


void LKM1638_write(unsigned char data);
unsigned char LKM1638_read(void);
void write_command(unsigned char cmd);
unsigned char read_button(void);
void write_segments(unsigned char led_num, int data);
void write_led(unsigned char num,unsigned char flag);
void write_all_leds(unsigned char LED_flag);
void write_hex_digit(int digit, int position);
int write_decimal(int digit, int position);
void write_decimal_4(int digit, int position);
void write_decimal_8(unsigned long digit);

// Hex segment codes for nos. 0-f
unsigned char digits[16] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F,0x77,0x7C,0x39,0x5E,0x79,0x71};

// ASCII to seven segment approximation
// from SPACE (0x20) to "`" (0x60) . Input string upper case only. Characters like K, X, V, W, Z or M show as a dash.
unsigned char ascii[] =
{
	0x00, 0x82, 0x21, 0x40, 0x40, 0x40, 0x40, 0x02, 0x39, 0x0f, 0x40, 0x40, 0x40, 0x40, 0x80, 0x40,
	0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07, 0x7f, 0x6f, 0x40, 0x40, 0x40, 0x48, 0x40, 0x53,
	0x40, 0x77, 0x7c, 0x39, 0x5e, 0x79, 0x71, 0x6f, 0x76, 0x06, 0x1e, 0x40, 0x38, 0x40, 0x54, 0x3f,
	0x73, 0x67, 0x50, 0x6d, 0x78, 0x3e, 0x40, 0x40, 0x40, 0x6e, 0x40, 0x39, 0x40, 0x0f, 0x40, 0x00,
	0x63
};

void LKM1638_write(unsigned char data)			//  write data (meant to be seen) to the LKM1638
{
	unsigned char i;
	DIO_OUT;
	for(i=0;i<8;i++)
	{
		CLK_low;
		if(data&0X01)
			DIO_high;
		else
			DIO_low;
		data>>=1;
		CLK_high;
		_delay_us(1);
	}
}

unsigned char LKM1638_read(void)					// read from the LKM1638
{
	unsigned char i;
	unsigned char temp=0;
	DIO_IN;
	_delay_us(1);
	for(i=0;i<8;i++)
	{
		temp>>=1;
		CLK_low;
		_delay_us(1);
		if(DIO_READ)
			temp|=0x80;
		CLK_high;
	}
	return temp;
}

void write_command(unsigned char cmd)		// write a command to the LKM1638
{
	STB_low;
	LKM1638_write(cmd);
	STB_high;
	_delay_us(1);
}

unsigned char read_button(void)
{
	unsigned char c[4],i,key_value=0;
	STB_low;
	LKM1638_write(0x42);
	_delay_us(1);
	for(i=0;i<4;i++)
		c[i]=LKM1638_read();
	STB_high;					//4ֽݺϳһֽ
	for(i=0;i<4;i++)
		key_value|=c[i]<<i;
	for(i=0;i<8;i++)
		if((0x01<<i)==key_value)
			break;
	return i;
}

// write_segments(Segment Number, ON=1 or OFF=0)
// Segments:
// 0b1000 00000		0x80	DECIMAL POINT

void write_segments(unsigned char led_num, int data)		//write segment byte to LED no. led_num, 1=segment on, 2=segment off

{
	write_command(0x44);
	STB_low;
	LKM1638_write(0xc0|led_num);
	LKM1638_write(data);
	STB_high;
}

void write_led(unsigned char num,unsigned char flag)	// Turn on one LED, 1st integer is LED number (0-7), 2nd is on (1) or off (0)
{
	if(flag)
		write_segments(2*num+1,1);
	else
		write_segments(2*num+1,0);
}

void write_all_leds(unsigned char LED_flag)		//Address all LEDs at once, using 8-bit integer where 1=on, 0=off
{
	unsigned char i;
	for(i=0;i<8;i++)
		{
			if(LED_flag&(1<<i))
				write_segments(2*i+1,1);
			else
				write_segments(2*i+1,0);
		}
}

void LKM1638_init(void)			// Initialize the LKM1638; call this first
{
	unsigned char i;
	write_command(0x89);				// reset
	write_command(0x40);				// send character data (0x00; clear the digits)
	STB_low;							// clock it in
	LKM1638_write(0xc0);
	for(i=0;i<16;i++)					// and clear
		LKM1638_write(0x00);
	STB_high;
}

void display_clr(void)	// blank the display
{
	int i;
	for (i=0; i<16; i++)
	{
		write_segments(i, 0);
	}
}

void write_str(char *string)	// write a ASCII string to the display, 8 characters maximum (longer will wrap around)
{
	int i;
	int j;
	for (i=0; i < (strlen(string)); i++)
	{
		j = (string[i] - 0x20);
		write_segments((i*2), ascii[j]);
	}
}

void write_hex_digit(int digit, int position)	// write a single hex digit at position (0-7)
{
	int temp;
	if (digit > 0x0f)	// bounds checking
	{
		temp=0;			// write a blank in invalid hex digit (>15)
	}
	else
	{
		temp=digits[digit];	// convert to hex via table
	}
	position = position << 1;	// align to use 7-seg, not LED
	write_segments(position, temp);	// and send it
}

int write_decimal(int digit, int position)	// write single digit at position 0-7 converting to decimal. Returns amount in excess of 9 (if you want it)
{
	int temp = digit;
	int carry = 0;
	while (temp > 9)	// did we give this routine a number larger than 9?
	{
		temp -=10;
		carry = carry + 10;
	}
	position = position << 1;	// align to use 7-seg, not LED
	write_segments(position, digits[temp]);	// convert to character via table and send it
	return(carry);	// return excess amount over 9
}

void write_decimal_4(int digit, int position)	// write a number less than 10000 converting to decimal at position 0 or 4 using four numerals
{
	int i, temp;
	i=(position + 4);

	while (i != position)	// run through 4 digits starting at rightmost digit
	{
		temp = write_decimal(digit, i-1);	// write least significant number furthest right
		i--;								// and move to next position leftwards
		digit = temp/10;					// remainder from 'write_decimal' by ten to call 'write_decimal' again
	}
}

void write_decimal_8(unsigned long digit)	// write a number converting to decimal at position 0 using four numerals (0 - 99,999,999)
{
	unsigned long int temp = 0;
	/*while (digit > 10000)
	{
		digit = digit - 10000;
		temp = temp + 1;
	} */
	temp = digit / 10000;
	digit = digit - (temp * 10000);
	write_decimal_4(digit, 4);
	write_decimal_4(temp, 0);
}
